From dbad2912cedeb8bf19dd5f34591951d7d67feeae Mon Sep 17 00:00:00 2001
From: Matthew Flatt <mflatt@racket-lang.org>
Date: Fri, 31 Oct 2014 07:46:35 -0600
Subject: [PATCH] fixups for compiling on Cygwin

---
 racket/collects/compiler/private/xform.rkt |  2 +-
 racket/src/configure                       |  1 -
 racket/src/foreign/foreign.c               |  3 +++
 racket/src/foreign/foreign.rktc            |  1 +
 racket/src/racket/configure.ac             |  1 -
 racket/src/racket/gc2/sighand.c            |  5 ++++-
 racket/src/racket/gc2/xform-mod.rkt        |  2 +-
 racket/src/racket/include/mzwin.def        |  3 ---
 racket/src/racket/include/mzwin3m.def      |  3 ---
 racket/src/racket/include/racket.exp       |  3 ---
 racket/src/racket/include/racket3m.exp     |  3 ---
 racket/src/racket/include/scheme.h         |  4 ++--
 racket/src/racket/include/schthread.h      |  2 +-
 racket/src/racket/sconfig.h                |  7 -------
 racket/src/racket/sgc/Makefile.in          |  6 +++++-
 racket/src/racket/sgc/sgc.h                |  2 +-
 racket/src/racket/src/eval.c               |  2 +-
 racket/src/racket/src/port.c               |  2 +-
 racket/src/racket/src/schemef.h            | 10 +++-------
 racket/src/racket/src/schemex.h            |  9 +++------
 racket/src/racket/src/schemex.inc          |  3 ---
 racket/src/racket/src/schemexm.h           |  3 ---
 racket/src/racket/src/setjmpup.c           |  9 +++++----
 23 files changed, 32 insertions(+), 54 deletions(-)

diff --git a/racket/collects/compiler/private/xform.rkt b/racket/collects/compiler/private/xform.rkt
index f609110..ff36e83 100644
--- a/racket/collects/compiler/private/xform.rkt
+++ b/racket/collects/compiler/private/xform.rkt
@@ -886,8 +886,8 @@
                ;; they either take one argument or no pointer arguments.
                ;; So we can ignore them:
                
-               
                __get_errno_ptr ; QNX preprocesses errno to __get_errno_ptr
+               __getreent ; Cygwin
 
                strlen cos cosl sin sinl exp expl pow powl log logl sqrt sqrtl atan2 atan2l
                isnan isinf fpclass _fpclass __fpclassify __fpclassifyf __fpclassifyl
diff --git a/racket/src/configure b/racket/src/configure
index 0d91837..48fa5f9 100755
--- a/racket/src/configure
+++ b/racket/src/configure
@@ -4584,7 +4584,6 @@ $as_echo "#define HAVE_STDINT_H 1" >>confdefs.h
     esac
     ;;
   cygwin*)
-    enable_cgcdefault="yes"
     MZINSTALLTARGET=unix-cygwin-install
     if test "${enable_shared}" = "yes" ; then
       ar_libtool_no_undefined=" -no-undefined"
diff --git a/racket/src/foreign/foreign.c b/racket/src/foreign/foreign.c
index b5be78d..5babfa2 100644
--- a/racket/src/foreign/foreign.c
+++ b/racket/src/foreign/foreign.c
@@ -267,6 +267,9 @@ static Scheme_Object *foreign_ffi_lib(int argc, Scheme_Object *argv[])
 #   ifdef __ANDROID__
     if (!name) handle = RTLD_DEFAULT; else
 #   endif /* __ANDROID__ */
+#   ifdef __CYGWIN32__
+    if (!name) { handle = RTLD_DEFAULT; null_ok = 1; } else
+#   endif /* __CYGWIN32__ */
     handle = dlopen(name, RTLD_NOW | (as_global ? RTLD_GLOBAL : RTLD_LOCAL));
 #   endif /* WINDOWS_DYNAMIC_LOAD */
     if (handle == NULL && !null_ok) {
diff --git a/racket/src/foreign/foreign.rktc b/racket/src/foreign/foreign.rktc
index f1ded27..e67332c 100755
--- a/racket/src/foreign/foreign.rktc
+++ b/racket/src/foreign/foreign.rktc
@@ -235,6 +235,7 @@ THREAD_LOCAL_DECL(static Scheme_Hash_Table *opened_libs);
         handle = LoadLibraryW(WIDE_PATH(name));
     }{
       @@IFDEF{__ANDROID__}{if (!name) handle = RTLD_DEFAULT; else}
+      @@IFDEF{__CYGWIN32__}{if (!name) { handle = RTLD_DEFAULT; null_ok = 1; } else}
       handle = dlopen(name, RTLD_NOW | (as_global ? RTLD_GLOBAL : RTLD_LOCAL));
     }
     if (handle == NULL && !null_ok) {
diff --git a/racket/src/racket/configure.ac b/racket/src/racket/configure.ac
index 71884cd..2895aa1 100644
--- a/racket/src/racket/configure.ac
+++ b/racket/src/racket/configure.ac
@@ -718,7 +718,6 @@ case "$host_os" in
     esac
     ;;    
   cygwin*)
-    enable_cgcdefault="yes"
     MZINSTALLTARGET=unix-cygwin-install
     if test "${enable_shared}" = "yes" ; then
       ar_libtool_no_undefined=" -no-undefined"
diff --git a/racket/src/racket/gc2/sighand.c b/racket/src/racket/gc2/sighand.c
index dff2fd5..6c46acd 100644
--- a/racket/src/racket/gc2/sighand.c
+++ b/racket/src/racket/gc2/sighand.c
@@ -162,7 +162,10 @@ void fault_handler(int sn, struct siginfo *si, void *ctx)
 #endif
 
 /* ========== Windows signal handler ========== */
-#if defined(_WIN32)
+#if defined(_WIN32) || defined(__CYGWIN32__)
+# if defined(__CYGWIN32__)
+#  include <windows.h>
+# endif
 LONG WINAPI fault_handler(LPEXCEPTION_POINTERS e) 
 {
   if ((e->ExceptionRecord->ExceptionCode == EXCEPTION_ACCESS_VIOLATION)
diff --git a/racket/src/racket/gc2/xform-mod.rkt b/racket/src/racket/gc2/xform-mod.rkt
index 310451f..1aa2691 100644
--- a/racket/src/racket/gc2/xform-mod.rkt
+++ b/racket/src/racket/gc2/xform-mod.rkt
@@ -65,7 +65,7 @@
       (call-with-output-file fn
         (lambda (out)
           (fprintf out "#define ~a ~a\n" (cadr m) (caddr m)))
-        'truncate/replace)
+        #:exists 'truncate/replace)
       (set! cpp (string-append cpp (format " -include ~a" fn)))]
      [else
       (set! cpp (string-append cpp " -D"
diff --git a/racket/src/racket/include/mzwin.def b/racket/src/racket/include/mzwin.def
index 793ff30..da4a740 100644
--- a/racket/src/racket/include/mzwin.def
+++ b/racket/src/racket/include/mzwin.def
@@ -96,9 +96,6 @@ EXPORTS
  scheme_set_can_break
  scheme_push_break_enable
  scheme_pop_break_enable
- scheme_with_stack_freeze
- scheme_frozen_run_some
- scheme_is_in_frozen_stack
  scheme_abort_continuation_no_dws
  scheme_call_with_composable_no_dws
  scheme_set_on_atomic_timeout
diff --git a/racket/src/racket/include/mzwin3m.def b/racket/src/racket/include/mzwin3m.def
index eec3094..7f679de 100644
--- a/racket/src/racket/include/mzwin3m.def
+++ b/racket/src/racket/include/mzwin3m.def
@@ -96,9 +96,6 @@ EXPORTS
  scheme_set_can_break
  scheme_push_break_enable
  scheme_pop_break_enable
- scheme_with_stack_freeze
- scheme_frozen_run_some
- scheme_is_in_frozen_stack
  scheme_abort_continuation_no_dws
  scheme_call_with_composable_no_dws
  scheme_set_on_atomic_timeout
diff --git a/racket/src/racket/include/racket.exp b/racket/src/racket/include/racket.exp
index 737fc80..88d46c8 100644
--- a/racket/src/racket/include/racket.exp
+++ b/racket/src/racket/include/racket.exp
@@ -93,9 +93,6 @@ scheme_pop_kill_action
 scheme_set_can_break
 scheme_push_break_enable
 scheme_pop_break_enable
-scheme_with_stack_freeze
-scheme_frozen_run_some
-scheme_is_in_frozen_stack
 scheme_abort_continuation_no_dws
 scheme_call_with_composable_no_dws
 scheme_set_on_atomic_timeout
diff --git a/racket/src/racket/include/racket3m.exp b/racket/src/racket/include/racket3m.exp
index 1905f9a..79f6007 100644
--- a/racket/src/racket/include/racket3m.exp
+++ b/racket/src/racket/include/racket3m.exp
@@ -93,9 +93,6 @@ scheme_pop_kill_action
 scheme_set_can_break
 scheme_push_break_enable
 scheme_pop_break_enable
-scheme_with_stack_freeze
-scheme_frozen_run_some
-scheme_is_in_frozen_stack
 scheme_abort_continuation_no_dws
 scheme_call_with_composable_no_dws
 scheme_set_on_atomic_timeout
diff --git a/racket/src/racket/include/scheme.h b/racket/src/racket/include/scheme.h
index 3bf595d..6de06b0 100644
--- a/racket/src/racket/include/scheme.h
+++ b/racket/src/racket/include/scheme.h
@@ -2128,8 +2128,8 @@ extern Scheme_Extension_Table *scheme_extension_table;
 # define MZ_FD_CLR(n, p) scheme_fdclr(p, n)
 # define MZ_FD_ISSET(n, p) scheme_fdisset(p, n)
 #else
-# define MZ_GET_FDSET(p, n) ((void *)(((fd_set *)p) + n))
-# define MZ_FD_ZERO(p) FD_ZERO((fd_set *)(p))
+# define MZ_GET_FDSET(p, n) ((void *)(((fd_set *)p) XFORM_OK_PLUS n))
+# define MZ_FD_ZERO(p) XFORM_HIDE_EXPR(FD_ZERO((fd_set *)(p)))
 # define MZ_FD_SET(n, p) FD_SET(n, (fd_set *)(p))
 # define MZ_FD_CLR(n, p) FD_CLR(n, (fd_set *)(p))
 # define MZ_FD_ISSET(n, p) FD_ISSET(n, (fd_set *)(p))
diff --git a/racket/src/racket/include/schthread.h b/racket/src/racket/include/schthread.h
index 54f1cdc..cdb270d 100644
--- a/racket/src/racket/include/schthread.h
+++ b/racket/src/racket/include/schthread.h
@@ -49,7 +49,7 @@ extern "C" {
 #endif
 
 /* Set up MZ_EXTERN for DLL build */
-#if (defined(__WIN32__) || defined(WIN32) || defined(_WIN32)) \
+#if (defined(__WIN32__) || defined(WIN32) || defined(_WIN32) || defined(__CYGWIN32__)) \
     && !defined(LINK_EXTENSIONS_BY_TABLE) \
     && !defined(SCHEME_EMBEDDED_NO_DLL)
 # define MZ_DLLIMPORT __declspec(dllimport)
diff --git a/racket/src/racket/sconfig.h b/racket/src/racket/sconfig.h
index 0aa3fff..fdab893 100644
--- a/racket/src/racket/sconfig.h
+++ b/racket/src/racket/sconfig.h
@@ -589,9 +589,6 @@
 
 # define DO_STACK_CHECK
 # define WINDOWS_FIND_STACK_BOUNDS
-/* Default stack size is 1MB, but we try to read
-   the actual size from the executable on startup: */
-# define WINDOWS_DEFAULT_STACK_SIZE 1048576
 
 # if !defined(_WIN64) || (_MSC_VER >= 1600)
 #  define USE_MZ_SETJMP
@@ -731,10 +728,6 @@
 
 # define REGISTER_POOR_MACHINE
 
-# ifndef AS_MSVC_EXTENSION
-#  define LINK_EXTENSIONS_BY_TABLE
-# endif
-
 # define MZ_USE_JIT_I386
 
 # define FLAGS_ALREADY_SET
diff --git a/racket/src/racket/sgc/Makefile.in b/racket/src/racket/sgc/Makefile.in
index dd1e45f..969c822 100644
--- a/racket/src/racket/sgc/Makefile.in
+++ b/racket/src/racket/sgc/Makefile.in
@@ -7,6 +7,10 @@ prefix = @prefix@
 exec_prefix = @exec_prefix@
 libdir = @libdir@
 
+# for version.mak:
+mainsrcdir = @srcdir@/../..
+@INCLUDEDEP@ @srcdir@/../version.mak
+
 CC = @CC@
 CXX = @CXX@
 RANLIB = @RANLIB@
@@ -32,7 +36,7 @@ test: $(OBJS) test.@LTO@
 gcobjects: $(OBJS)
 
 sgc.@LTO@: $(srcdir)/sgc.c $(srcdir)/autostat.inc $(srcdir)/collect.inc $(srcdir)/../utils/splay.c
-	$(CC) $(CFLAGS) $(CPPFLAGS) @OPTIONS@ -I.. -c $(srcdir)/sgc.c -o sgc.@LTO@
+	$(CC) $(CFLAGS) $(CPPFLAGS) @OPTIONS@ -DSGC_EXPORTS -I.. -c $(srcdir)/sgc.c -o sgc.@LTO@
 
 test.@LTO@: $(srcdir)/test.c
 	$(CC) $(CFLAGS) $(CPPFLAGS) @OPTIONS@ -c $(srcdir)/test.c -o test.@LTO@
diff --git a/racket/src/racket/sgc/sgc.h b/racket/src/racket/sgc/sgc.h
index 0dae616..2dffb99 100644
--- a/racket/src/racket/sgc/sgc.h
+++ b/racket/src/racket/sgc/sgc.h
@@ -7,7 +7,7 @@ extern "C" {
 
 #define GC_PTR void*
 
-#ifdef WIN32
+#if defined(WIN32) || defined(__CYGWIN32__)
 # ifdef SGC_EXPORTS
 #  define SGC_EXTERN __declspec(dllexport)
 # else
diff --git a/racket/src/racket/src/eval.c b/racket/src/racket/src/eval.c
index bf698d4..d7bb626 100644
--- a/racket/src/racket/src/eval.c
+++ b/racket/src/racket/src/eval.c
@@ -543,6 +543,7 @@ static uintptr_t adjust_stack_base(uintptr_t bnd) {
 #ifdef WINDOWS_FIND_STACK_BOUNDS
 intptr_t find_exe_stack_size()
 {
+# define WINDOWS_DEFAULT_STACK_SIZE 1048576
   intptr_t sz = WINDOWS_DEFAULT_STACK_SIZE;
   wchar_t *fn;
   DWORD len = 1024;
@@ -580,7 +581,6 @@ intptr_t find_exe_stack_size()
 		/* Skip to PE32[+] header's stack reservation value: */
 		if (SetFilePointer(fd, pos + 20 + 4 + 72, NULL, FILE_BEGIN)
 		    != INVALID_SET_FILE_POINTER) {
-		  mzlonglong lsz;
 		  if (kind == 0x10b) {
 		    /* PE32: 32-bit stack size: */
 		    int ssz;
diff --git a/racket/src/racket/src/port.c b/racket/src/racket/src/port.c
index 4229ef7..7f35778 100644
--- a/racket/src/racket/src/port.c
+++ b/racket/src/racket/src/port.c
@@ -1212,7 +1212,7 @@ void scheme_fdzero(void *fd)
   scheme_init_fdset_array(fd, 1);
 # else
 #  if defined(FILES_HAVE_FDS) || defined(USE_TCP)
-  FD_ZERO((fd_set *)fd);
+  XFORM_HIDE_EXPR(FD_ZERO((fd_set *)fd));
 #  endif
 # endif
 }
diff --git a/racket/src/racket/src/schemef.h b/racket/src/racket/src/schemef.h
index 0a5e3ed..75a0366 100644
--- a/racket/src/racket/src/schemef.h
+++ b/racket/src/racket/src/schemef.h
@@ -201,12 +201,8 @@ MZ_EXTERN void scheme_set_can_break(int on);
 MZ_EXTERN void scheme_push_break_enable(Scheme_Cont_Frame_Data *cframe, int on, int pre_check);
 MZ_EXTERN void scheme_pop_break_enable(Scheme_Cont_Frame_Data *cframe, int post_check);
 
-MZ_EXTERN int scheme_with_stack_freeze(Scheme_Frozen_Stack_Proc wha_f, void *wha_data);
-MZ_EXTERN int scheme_frozen_run_some(Scheme_Frozen_Stack_Proc do_f, void *do_data, int run_msecs);
-MZ_EXTERN int scheme_is_in_frozen_stack();
-
-MZ_EXTERN Scheme_Object *scheme_abort_continuation_no_dws (Scheme_Object *pt, Scheme_Object *v);
-MZ_EXTERN Scheme_Object *scheme_call_with_composable_no_dws (Scheme_Object *proc, Scheme_Object *pt);
+MZ_EXTERN Scheme_Object *scheme_abort_continuation_no_dws(Scheme_Object *pt, Scheme_Object *v);
+MZ_EXTERN Scheme_Object *scheme_call_with_composable_no_dws(Scheme_Object *proc, Scheme_Object *pt);
 
 MZ_EXTERN Scheme_On_Atomic_Timeout_Proc scheme_set_on_atomic_timeout(Scheme_On_Atomic_Timeout_Proc p);
 
@@ -1229,5 +1225,5 @@ MZ_EXTERN void *scheme_register_process_global(const char *key, void *val);
 MZ_EXTERN Scheme_Object *scheme_malloc_key(void);
 MZ_EXTERN void scheme_free_key(Scheme_Object *k);
 
-MZ_EXTERN void* scheme_jit_find_code_end(void *p);
+MZ_EXTERN void *scheme_jit_find_code_end(void *p);
 MZ_EXTERN void scheme_jit_now(Scheme_Object *f);
diff --git a/racket/src/racket/src/schemex.h b/racket/src/racket/src/schemex.h
index e82bd91..97dcfd9 100644
--- a/racket/src/racket/src/schemex.h
+++ b/racket/src/racket/src/schemex.h
@@ -145,11 +145,8 @@ void (*scheme_pop_kill_action)();
 void (*scheme_set_can_break)(int on);
 void (*scheme_push_break_enable)(Scheme_Cont_Frame_Data *cframe, int on, int pre_check);
 void (*scheme_pop_break_enable)(Scheme_Cont_Frame_Data *cframe, int post_check);
-int (*scheme_with_stack_freeze)(Scheme_Frozen_Stack_Proc wha_f, void *wha_data);
-int (*scheme_frozen_run_some)(Scheme_Frozen_Stack_Proc do_f, void *do_data, int run_msecs);
-int (*scheme_is_in_frozen_stack)();
-Scheme_Object *scheme_abort_continuation_no_dws;
-Scheme_Object *scheme_call_with_composable_no_dws;
+Scheme_Object *(*scheme_abort_continuation_no_dws)(Scheme_Object *pt, Scheme_Object *v);
+Scheme_Object *(*scheme_call_with_composable_no_dws)(Scheme_Object *proc, Scheme_Object *pt);
 Scheme_On_Atomic_Timeout_Proc (*scheme_set_on_atomic_timeout)(Scheme_On_Atomic_Timeout_Proc p);
 /*========================================================================*/
 /*                              error handling                            */
@@ -1001,7 +998,7 @@ Scheme_Hash_Table *(*scheme_get_place_table)(void);
 void *(*scheme_register_process_global)(const char *key, void *val);
 Scheme_Object *(*scheme_malloc_key)(void);
 void (*scheme_free_key)(Scheme_Object *k);
- (*scheme_jit_find_code_end)(void *p);
+void *(*scheme_jit_find_code_end)(void *p);
 void (*scheme_jit_now)(Scheme_Object *f);
 #ifndef SCHEME_EX_INLINE
 } Scheme_Extension_Table;
diff --git a/racket/src/racket/src/schemex.inc b/racket/src/racket/src/schemex.inc
index 8f4850d..3ef9946 100644
--- a/racket/src/racket/src/schemex.inc
+++ b/racket/src/racket/src/schemex.inc
@@ -102,9 +102,6 @@
   scheme_extension_table->scheme_set_can_break = scheme_set_can_break;
   scheme_extension_table->scheme_push_break_enable = scheme_push_break_enable;
   scheme_extension_table->scheme_pop_break_enable = scheme_pop_break_enable;
-  scheme_extension_table->scheme_with_stack_freeze = scheme_with_stack_freeze;
-  scheme_extension_table->scheme_frozen_run_some = scheme_frozen_run_some;
-  scheme_extension_table->scheme_is_in_frozen_stack = scheme_is_in_frozen_stack;
   scheme_extension_table->scheme_abort_continuation_no_dws = scheme_abort_continuation_no_dws;
   scheme_extension_table->scheme_call_with_composable_no_dws = scheme_call_with_composable_no_dws;
   scheme_extension_table->scheme_set_on_atomic_timeout = scheme_set_on_atomic_timeout;
diff --git a/racket/src/racket/src/schemexm.h b/racket/src/racket/src/schemexm.h
index 362c47c..aaf2ede 100644
--- a/racket/src/racket/src/schemexm.h
+++ b/racket/src/racket/src/schemexm.h
@@ -102,9 +102,6 @@
 #define scheme_set_can_break (scheme_extension_table->scheme_set_can_break)
 #define scheme_push_break_enable (scheme_extension_table->scheme_push_break_enable)
 #define scheme_pop_break_enable (scheme_extension_table->scheme_pop_break_enable)
-#define scheme_with_stack_freeze (scheme_extension_table->scheme_with_stack_freeze)
-#define scheme_frozen_run_some (scheme_extension_table->scheme_frozen_run_some)
-#define scheme_is_in_frozen_stack (scheme_extension_table->scheme_is_in_frozen_stack)
 #define scheme_abort_continuation_no_dws (scheme_extension_table->scheme_abort_continuation_no_dws)
 #define scheme_call_with_composable_no_dws (scheme_extension_table->scheme_call_with_composable_no_dws)
 #define scheme_set_on_atomic_timeout (scheme_extension_table->scheme_set_on_atomic_timeout)
diff --git a/racket/src/racket/src/setjmpup.c b/racket/src/racket/src/setjmpup.c
index 7f59fe0..c339f83 100644
--- a/racket/src/racket/src/setjmpup.c
+++ b/racket/src/racket/src/setjmpup.c
@@ -673,12 +673,13 @@ void scheme_reset_jmpup_buf(Scheme_Jumpup_Buf *b)
    is fragile, because it's not well defined whether the compiler
    will generate frame-pointer setup; use mzsj86g.S, instead. */
 
-#ifdef __MINGW32__
-# if __OPTIMIZE__ > 0
-#  define NEED_STACK_FRAME_SETUP
-# endif
+#if (__OPTIMIZE__ > 0) || defined(MZ_XFORM)
+# define NEED_STACK_FRAME_SETUP
 #endif
 
+MZ_DO_NOT_INLINE(int scheme_mz_setjmp(mz_pre_jmp_buf b));
+MZ_DO_NOT_INLINE(void scheme_mz_longjmp(mz_pre_jmp_buf b, int v));
+
 int scheme_mz_setjmp(mz_pre_jmp_buf b)
 {
 #ifdef NEED_STACK_FRAME_SETUP
-- 
1.9.3 (Apple Git-50)

